
//--------------------------------------------------------------------------------------------------
#include "shared.fxh"

//--------------------------------------------------------------------------------------------------
texture Texture0;

//--------------------------------------------------------------------------------------------------
sampler g_DiffuseSampler = sampler_state
{ 
    Texture = ( Texture0 );
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

//--------------------------------------------------------------------------------------------------
// VertexShader I/O
//--------------------------------------------------------------------------------------------------
struct VS_INPUT
{
	float3 vPositionOS		: POSITION;				// object space position
	float3 vNormalOS		: NORMAL0;				// object space normal
	float2 vTexCoord		: TEXCOORD0;			// texture coordinates
};
struct VS_OUTPUT
{
	float4 vPositionPS		: POSITION;				// projection space position
	float2 vTexCoord		: TEXCOORD0;			// texture coordinates
	float3 vPositionOS		: TEXCOORD1;			// object space position
	float3 vNormalOS		: TEXCOORD2;			// object space normal
};	
//--------------------------------------------------------------------------------------------------
float4 psMain( VS_OUTPUT i ) : COLOR0
{
	float4 vLightPos = float4( g_aLights[0].vPositionWS, 1 );
	float4 vEyePos = vLightPos;
	float3 vLightDir = normalize( vLightPos - i.vPositionOS );
	float  fLightDistance = length( vLightPos - i.vPositionOS );
	float4 vColor = 0;

	// ambient lighting
	float fAmbientReflectivity = 1;
	float3 vAmbientMaterial = tex2D( g_DiffuseSampler, i.vTexCoord );
	vColor.rgb += vAmbientMaterial * g_vAmbientLight * fAmbientReflectivity;

	// diffuse lighting (lambert)
	float fDiffuseReflectivity = max( dot( i.vNormalOS, vLightDir ), 0 );
	float3 vDiffuseLight = g_aLights[0].vColor;
	float3 vDiffuseMaterial = tex2D( g_DiffuseSampler, i.vTexCoord ) * 0.9;
	vColor.rgb += vDiffuseMaterial * vDiffuseLight * fDiffuseReflectivity;

	// specular lighting (phong)
	float3 vReflection = -reflect( vLightDir, i.vNormalOS );
	float fSpecularReflectivity = pow( max( dot( vReflection, vEyePos ), 2 ), 2 );
	float3 vSpecularLight = g_aLights[0].vColor;
	float3 vSpecularMaterial = 0;
	vColor.rgb += vSpecularMaterial * vSpecularLight * fSpecularReflectivity;


	return vColor;
}
//--------------------------------------------------------------------------------------------------
VS_OUTPUT vsMain( VS_INPUT i )
{
	// preshader->
	const float4x4 mViewProjection = mul( g_mView, g_mProjection );
	const float4x4 mWorldViewProjection = mul( g_mWorld, mViewProjection );
	// <-preshader

	VS_OUTPUT o;

	o.vTexCoord = i.vTexCoord;
	o.vPositionPS = mul( float4( i.vPositionOS, 1 ), mWorldViewProjection );
	o.vPositionOS = i.vPositionOS;
	o.vNormalOS = mul( i.vNormalOS, g_mWorld );		
	return o;
}
//--------------------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------------------
technique Standard
{
    pass P0
    {   
        VertexShader = compile vs_2_0 vsMain();
        PixelShader  = compile ps_2_0 psMain();
		ZEnable = true;
        ZWriteEnable = true;
    }
}
//--------------------------------------------------------------------------------------------------
